"""
======================
Script to Explore SVMs
======================

Simple script to explore SVM training with varying C

Example adapted from scikit_learn documentation by Eric Eaton, 2014

"""
print(__doc__)

import numpy as np
import matplotlib.pyplot as plt
from sklearn import svm


# load the data
filename = 'data/svmData.dat'
allData = np.loadtxt(filename, delimiter=',')

X = allData[:, :-1]
Y = allData[:, -1]


C = 0.01

# train the SVM
print("Training the SVM")
clf = svm.SVC(C=C, kernel='linear')
clf.fit(X, Y)

print("")
print("Testing the SVM")

h = .02  # step size in the mesh

# Plot the decision boundary. For that, we will assign a color to each
# point in the mesh [x_min, m_max]x[y_min, y_max].
x_min, x_max = X[:, 0].min() - 1, X[:, 0].max() + 1
y_min, y_max = X[:, 1].min() - 1, X[:, 1].max() + 1
xx, yy = np.meshgrid(np.arange(x_min, x_max, h), np.arange(y_min, y_max, h))
Z = clf.predict(np.c_[xx.ravel(), yy.ravel()])

# Put the result into a color plot
Z = Z.reshape(xx.shape)
plt.pcolormesh(xx, yy, Z, cmap='Paired')

# Plot also the training points
plt.scatter(X[:, 0], X[:, 1], c=Y, cmap='Paired', edgecolors="black")
plt.title('SVM decision surface with C = '+str(C))
plt.axis('tight')
plt.show()
